@@ -1,16 +1,11 @@
 from dataclasses import dataclass
 from datetime import datetime
 from enum import Enum
-from typing import List, Any, Dict
+from typing import Any, Dict, List
 
 from pydantic import BaseModel, Field
 
 
-class AvailableBlockchainType(Enum):
-    ETHEREUM = "ethereum"
-    POLYGON = "polygon"
-
-
 class StatsUpdateRequest(BaseModel):
     dashboard_id: str
     timescales: List[str]
@@ -54,3 +49,11 @@ class QueryDataUpdate(BaseModel):
     file_type: str
     query: str
     params: Dict[str, Any] = Field(default_factory=dict)
+
+
+class TokenURIs(BaseModel):
+    token_id: str
+    token_uri: str
+    block_number: str
+    block_timestamp: str
+    address: str
